# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests, json
from django.conf import settings
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth import authenticate
from dss.Serializer import serializer

from .models import UserProfile
from .utils import encode, decode

JSCODE2SESSION_URL = 'https://api.weixin.qq.com/sns/jscode2session?grant_type=authorization_code'
APPID = settings.WEIXIN_APPID
APPSECRET = settings.WEIXIN_APPSECRET


# 接受小程序发送的code，然后从微信服务器换取用户的微信身份ID
def login(request):
    code = request.GET.get('code')
    url = JSCODE2SESSION_URL + '&appid=' + APPID + '&secret=' + APPSECRET + '&js_code=' + code
    # login(request, user)
    r = requests.get(url).json()
    openid = r['openid']
    session_key = r['session_key']

    try:
        userprofile = UserProfile.objects.get(wxapp_openid=openid)
        # setattr(userprofile, 'user', userprofile.user)
        # result = serializer(data=userprofile, output_type='json', datetime_format='string',include_attr=('user_id',"student_id","sn","room_id","user_class","start_date","expire_date","preuser","avatar","is_pause"))
        # result_dict = json.loads(result)
        # result_dict["username"]=userprofile.user.username
        # result_dict["name"]=userprofile.user.first_name
        # result_dict["days"]=userprofile.get_expire_days()
        # result_dict["is_unlimited"]=userprofile.user.is_unlimited()
        # result_dict["get_sn"]=userprofile.get_sn()
        # result_dict["coin"]=userprofile.user.get_coin_count()

        return JsonResponse(data={
            'errCode': 0,
            'token': encode(openid),
            'errMsg': "成功",
            "userInfo": userprofile.to_json()
        })
    except UserProfile.DoesNotExist:
        # 用户还未绑定微信账号
        return JsonResponse(data={
            'errCode': 1,
            'token': encode(openid),
            'errMsg': "未绑定账户"
        })
    except Exception as e:
        print(e)
        return JsonResponse(data={
            'errCode': 2,
            'errMsg': "发生错误"
        })


# 绑定网站的用户和微信的用户
@csrf_exempt
def user_bind(request):
    username = request.POST.get('username')
    password = request.POST.get('password')
    token = request.POST.get('token')
    print(username, password, token)
    user = authenticate(username=username, password=password)
    if user is not None and user.is_active == True:
        openid = decode(token)

        if openid:
            # existing_weixininfo = user.weixininfo_set.all()
            existing_users = UserProfile.objects.filter(wxapp_openid=openid)
            if existing_users.exists():
                # 不能重复绑定
                return JsonResponse(data={
                    'errCode': 3,
                    'errMsg': "当前微信已绑定过用户，不能重复绑定"
                })
            else:
                # 绑定成功
                user.userprofile.wxapp_openid = openid
                user.userprofile.save()
                result = serializer(data=user.userprofile, output_type='json', datetime_format='string', include_attr=(
                'user_id', "student_id", "sn", "room_id", "user_class", "start_date", "expire_date", "preuser",
                "avatar", "is_pause"))
                result_dict = json.loads(result)
                result_dict["username"] = user.username
                result_dict["name"] = user.first_name
                result_dict["days"] = user.userprofile.get_expire_days()
                result_dict["is_unlimited"] = user.userprofile.user.is_unlimited()
                result_dict["get_sn"] = user.userprofile.get_sn()
                result_dict["coin"] = user.userprofile.user.get_coin_count()
                return JsonResponse(data={
                    'errCode': 0,
                    'errMsg': "绑定成功",
                    'userInfo': user.userprofile.to_json()

                })
        else:
            # token已过期
            return JsonResponse(data={
                'errCode': 2,
                'errMsg': "token已过期"
            })
    else:
        # 用户不存在
        return JsonResponse(data={
            'errCode': 1,
            'errMsg': "用户名或密码错误"
        })

# 网站的用户和微信的用户解绑
@csrf_exempt
def user_unbind(request):
    # token = request.POST.get('token')
    # openid = decode(token)
    # existing_user = UserProfile.objects.get(wxapp_openid=openid)
    existing_user=request.user
    if existing_user:
        print(existing_user.userprofile.wxapp_openid)
        existing_user.userprofile.wxapp_openid = None
        existing_user.userprofile.save()
        return JsonResponse(data={
            'errCode': 0,
            'errMsg': "解绑成功"
        })
    else:
        return JsonResponse(data={
            'errCode': 1,
            'errMsg': "错误"
        })



# -*- coding:utf-8 -*-
__author__ = 'bee'

from django import forms
from django.contrib.auth.models import User, Group

from .models import UserProfile,UserProfileParentRelation, UserClass, UserLeaveRecord, USER_LEAVE_TYPE_CHOICES
from django.forms.models import inlineformset_factory
from .validators import user_sn_validator


class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ["first_name"]


class UserSearchForm(forms.Form):
    status_list = ((0, '全部'), (1, '正常'), (999, '请假'),(2, '暂停'), (3, '禁用'))
    status = forms.ChoiceField(choices=status_list, label='状态', required=False)
    group = forms.ModelChoiceField(queryset=Group.objects.all(), label='用户组', required=False)
    student_id = forms.CharField(label='学号', required=False)
    first_name = forms.CharField(label='用户姓名', required=False)
    start_expire_date = forms.CharField(label='结课日期', required=False)
    end_expire_date = forms.CharField(label='至', required=False)


    # class Meta:
    #     model = User
    #     fields = ["student_id","first_name"]


class UserProfileForm(forms.ModelForm):
    class Meta:
        model = UserProfile
        fields = ['start_date', "user_class"]

class UserProfileParentsForm(forms.Form):
    parent = forms.ModelChoiceField(queryset=User.objects.filter(groups__name__in=['家长助教']),label='家长助教',required=False)
    # class Meta:
    #     model = UserProfileParentRelation
    #     fields = ['parent']

profile_inline_formset = inlineformset_factory(User, UserProfile, form=UserProfileForm, can_delete=False)


class UserClassForm(forms.ModelForm):
    assistant_queryset = User.objects.filter(groups__name__in=['助教'])
    assistant = forms.ModelChoiceField(queryset=assistant_queryset, label='助教', required=False)

    class Meta:
        model = UserClass
        fields = ["name", 'assistant']

class UserClassSearchForm(forms.ModelForm):
    class_name = forms.CharField(label='班级名称', required=False)
    assistant_name = forms.CharField(label='助教姓名', required=False)

    class Meta:
        model = UserClass
        fields = ["class_name", 'assistant_name']

class UserGroupForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ['groups']


class UserSNForm(forms.ModelForm):
    sn = forms.CharField(label='缦客号', validators=[user_sn_validator])

    class Meta:
        model = UserProfile
        fields = ['sn']


# 请假/延期
class UserLeaveRecordForm(forms.ModelForm):
    type_choices = ((1, "请假"), (3, "延期"))
    type = forms.ChoiceField(choices=type_choices, label='类型')

    class Meta:
        model = UserLeaveRecord
        fields = ['type', "start", "end", "info"]


# 请假/延期
class UserLeaveRecordCancelForm(forms.ModelForm):
    type_choices = ((2, "销假"),)
    type = forms.ChoiceField(choices=type_choices, label='类型')

    class Meta:
        model = UserLeaveRecord
        fields = ['type', "start", "info"]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'
from django.conf.urls import include, url
from django.contrib.auth import views as auth_views

from . import wxapp


urlpatterns = [
    url(r'^login$', wxapp.login),
    url(r'^user/bind$', wxapp.user_bind),
    url(r'^user/unbind$', wxapp.user_unbind),
]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

import datetime
from .models import User, UserLeaveStatus,LOCAL_TIMEZONE


# 获取一个助教，所教的所有学生
def get_teach_users(assistant):
    users = User.objects.filter(userprofile__user_class__assistant=assistant)
    return users


# 获取一个学生是否请假的状态
def get_user_leave_status(user):
    record_list = UserLeaveStatus.objects.filter(user=user, status=1)
    if record_list.exists():
        return 1
    else:
        return 0


# 更新所有请假学生的状态
def update_user_leave_status():
    record_list = UserLeaveStatus.objects.filter(status__in=[0, 1, 2])  # 未处理或请假中
    for record in record_list:
        record.update_status()


# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-22 08:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0017_userprofile_is_pause'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserSN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.IntegerField(unique=True, verbose_name='\u8d77')),
                ('end', models.IntegerField(unique=True, verbose_name='\u8d77')),
                ('is_used', models.BooleanField(default=False, verbose_name='\u662f\u5426\u6b63\u5728\u4f7f\u7528')),
            ],
        ),
        migrations.AlterModelOptions(
            name='userprofile',
            options={'ordering': ['-created_at'], 'permissions': (('can_manage', '\u53ef\u4ee5\u8bbf\u95ee\u540e\u53f0\u7ba1\u7406'), ('can_change_user_group', '\u53ef\u4ee5\u4fee\u6539\u7528\u6237\u7ec4'), ('can_change_user_sn', '\u53ef\u4ee5\u4fee\u6539sn\u53f7'), ('reset_user_password', '\u53ef\u4ee5\u91cd\u7f6e\u7528\u6237\u5bc6\u7801'), ('view_all_users', '\u53ef\u4ee5\u67e5\u770b\u6240\u6709\u7528\u6237'), ('view_manage_users', '\u53ef\u4ee5\u67e5\u770b\u7ba1\u7406\u7684\u7528\u6237'), ('view_teach_users', '\u53ef\u4ee5\u67e5\u770b\u6559\u7684\u7528\u6237'))},
        ),
        migrations.AddField(
            model_name='userprofile',
            name='sn',
            field=models.IntegerField(null=True, unique=True, verbose_name='\u7edf\u4e00\u7f26\u5ba2\u53f7'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'zhangyue'

import os
from django.core.exceptions import ValidationError
from .models import UserSN,UserProfile


def user_sn_validator(value):
    sn = int(value)
    sn_list = UserSN.objects.filter(is_used=True).order_by('start')
    if not sn_list.exists():
        raise ValidationError(u'缦客号区间设置错误')
    _sn = sn_list.first()
    end = _sn.end
    max_sn = UserProfile.get_max_sn()
    if sn < end and sn > max_sn:
        raise ValidationError(u'超过最大值')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-06-30 09:58
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0025_auto_20190630_1540'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='userprofile',
            options={'ordering': ['-created_at'], 'permissions': (('can_manage', '\u53ef\u4ee5\u8bbf\u95ee\u540e\u53f0\u7ba1\u7406'), ('can_change_user_group', '\u53ef\u4ee5\u4fee\u6539\u7528\u6237\u7ec4'), ('can_change_user_sn', '\u53ef\u4ee5\u4fee\u6539sn\u53f7'), ('can_change_user_status', '\u53ef\u4ee5\u4fee\u6539\u7528\u6237\u72b6\u6001'), ('can_change_user_parent', '\u53ef\u4ee5\u4fee\u6539\u7528\u6237\u7684\u5bb6\u957f\u52a9\u6559'), ('reset_user_password', '\u53ef\u4ee5\u91cd\u7f6e\u7528\u6237\u5bc6\u7801'), ('view_all_users', '\u53ef\u4ee5\u67e5\u770b\u6240\u6709\u7528\u6237'), ('view_manage_users', '\u53ef\u4ee5\u67e5\u770b\u7ba1\u7406\u7684\u7528\u6237'), ('view_teach_users', '\u53ef\u4ee5\u67e5\u770b\u6559\u7684\u7528\u6237'), ('view_child_users', '\u53ef\u4ee5\u67e5\u770b\u4eb2\u5b50\u7528\u6237'))},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-06-30 07:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0024_userparentrelation'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='UserParentRelation',
            new_name='UserProfileParentRelation',
        ),
        migrations.AddField(
            model_name='userprofile',
            name='parents',
            field=models.ManyToManyField(through='bee_django_user.UserProfileParentRelation', to='bee_django_user.UserProfile'),
        ),
        migrations.AlterField(
            model_name='userprofileparentrelation',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='parent', to='bee_django_user.UserProfile', verbose_name='\u5bb6\u957f'),
        ),
        migrations.AlterField(
            model_name='userprofileparentrelation',
            name='student',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='student', to='bee_django_user.UserProfile', verbose_name='\u5b66\u751f'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-04-08 03:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0016_userprofile_avatar'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='is_pause',
            field=models.BooleanField(default=False, verbose_name='\u6682\u505c'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-06-21 14:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0022_auto_20190612_1746'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='wxapp_openid',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u5fae\u4fe1\u5c0f\u7a0b\u5e8fopen id'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-31 06:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0013_userprofile_is_leave'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userprofile',
            name='is_leave',
        ),
    ]
